#!/bin/perl
#############################################################################################################
# Chew's Website-Generator
#############################################################################################################
#
# Date        Version   Author         Changes
# 27.02.2004  1.0       Leonard Chew   Creation
# 22.11.2004  1.0.1     Leonard Chew   Added the 'INCLUDE_ORIG' Parameter (default = false) so that the user
#                                      is able to exclude the original picture. Recommenden when uploading
#                                      the website to a webserver with 'limited' discspace.
#
# This program creates a full website containing:
#   - index.html       (main html-page)
#   - tab<n>.html      (thumbnail-html-pages, depending on number of pics)
#   - folder 'thumbs'  (folder with jpg-thumbnails)
#   - folder 'web'     (folder with web-sized jpg-pics and corresponding html-pages for navigation)
#   - folder 'pics'    (folder with full-sized jpg-pics)
#   - folder 'movies'  (folder with avi-movies)
#   - folder 'icons'   (folder with needed icons, such as background picture)
#
# Prerequisites:
#   - Perl interpreter (on windows: recommended ActivePerl)
#       download at http://www.activestate.com/Products/Download/Get.plex?id=ActivePerl&_x=1
#   - Perl-Module Image::Magick
#       download Module at http://www.imagemagick.org/www/archives.html?
#   - Folder 'templates' (in same directory as this script) containing the html-template files
#   - Folder 'icons'     (in same directory as this script) containing the website-icons
#   - Configuration-File 'config.txt' (in same directory as this script)
#   - minimum one jpg- or avi-file in parent directory (= directory from which the website is generated)
#
############################################################################################################


use POSIX;
use File::Copy;
use Image::Magick;

# read configuration-values of config-file:
open(F_CONFIG,"./config.txt") || die "error in reading configuration-file";
while (<F_CONFIG>)
{
  $_ =~ s/\#.*//;
  if ($_ =~ /.+=.+/)
  {
     $_ =~ s/=/ /;
     ($KEY,$VALUE) = split(/ +/,$_);
     $CONFIG{$KEY} = $VALUE;
  }
}
close(F_CONFIG);


$NUMBER_COLS  = $CONFIG{"NUMBER_OF_COLUMNS"};   # Parameter of config file
$NUMBER_ROWS  = $CONFIG{"NUMBER_OF_ROWS"};      # Parameter of config file
$THUMB_SIZE   = $CONFIG{"THUMBNAIL_SIZE"};      # Parameter of config file
$PIC_SIZE     = $CONFIG{"PIC_SIZE"};            # Parameter of config file
$WAIT_TIME    = $CONFIG{"WAIT_TIME"} * 1000;    # Parameter of config file
$INLCUDE_ORIG = $CONFIG{"INLCUDE_ORIG"};        # Parameter of config file

$PICS_IN_PAGE = $NUMBER_COLS * $NUMBER_ROWS;

# Read Website-Source (Pics and Movies) #
#########################################
opendir(SRC_DIR,"..") || die "Parent directory not found";

$jpg_index = 1;
$avi_index = 0;
$TITLE1       = '';
$TITLE2       = '';
$WIDTH_SUBST  = '';

while ($FILENAME = readdir(SRC_DIR))
{
   if ($FILENAME =~ /\.[jJ][pP].?[gG]$/ )
   {
      $JPG[$jpg_index] = $FILENAME;
      $JPG_RENAMED[$jpg_index] = $FILENAME;
      $JPG_RENAMED[$jpg_index] =~ s/ /_/g;
      $JPG_RENAMED[$jpg_index] =~ s//ae/g;
      $JPG_RENAMED[$jpg_index] =~ s//oe/g;
      $JPG_RENAMED[$jpg_index] =~ s//ue/g;

      $HTML_PIC[$jpg_index] = $JPG_RENAMED[$jpg_index];
      $HTML_PIC[$jpg_index] =~ s/[jJ][pP].?[gG]$/html/;
      $jpg_index++;
   }

   if ($FILENAME =~ /\.[aA][vV][iI]$/ )
   {
      $AVI[$avi_index] = $FILENAME;
      $avi_index = $avi_index + 1;
   }
}

if ( $jpg_index == 1 && $avi_index == 0)
{
   print("no jpg or avi files found, exiting...\n");
   exit;
}

if ( $jpg_index > 1 )
{
   $TITLE1 = 'Pictures';
}

if ( $avi_index > 0 )
{
   $TITLE2 = 'Movies';
}

if ( $avi_index > 0 && $jpg_index > 1 )
{
   $WIDTH_SUBST = 'width="40"';
}

$NR_OF_PICS = $jpg_index - 1;
$NR_OF_THUMB_PAGES = ceil($NR_OF_PICS / $PICS_IN_PAGE);

# Generate Website #
####################
print ("generating Website...\n");

&create_defaults();
&create_html();

closedir(SRC_DIR);


# Sub-Procedures #
##################

# create the non-html default files (icons, thumbnails, pics)
sub create_defaults
{
   mkdir("../Website",0755);

   # copy all icons:
   mkdir("../Website/icons",0755);
   opendir(ICON_DIR,"./icons");
   while($icon = readdir(ICON_DIR))
   {
      copy("./icons/$icon","../Website/icons/$icon") if $icon =~ /.../;
   }
   closedir(ICON_DIR);

   # copy all libraries (javascript, css)
   mkdir("../Website/lib",0755);
   opendir(LIB_DIR,"./lib");
   while($lib = readdir(LIB_DIR))
   {
      copy("./lib/$lib","../Website/lib/$lib") if $lib =~ /.../;
   }
   closedir(LIB_DIR);

   # copy pics and thumbnails into Website-Folders:

   # only include the original pictures if configured:
   if ($INLCUDE_ORIG =~ /TRUE/ )
   {
      mkdir("../Website/pics",0755);
   }

   mkdir("../Website/thumbs",0755);
   mkdir("../Website/web",0755);
   for($i = 1; $i<=$NR_OF_PICS; $i++)
   {
      print "processing: '$JPG[$i]'\n";

      # only include the original pictures if configured:
      if ($INLCUDE_ORIG =~ /TRUE/ )
      {
         copy("../$JPG[$i]","../Website/pics/$JPG_RENAMED[$i]");
      }

      &copy_to_thumbnail($THUMB_SIZE, "../$JPG[$i]","../Website/thumbs/$JPG_RENAMED[$i]");
      &copy_to_thumbnail($PIC_SIZE, "../$JPG[$i]","../Website/web/$JPG_RENAMED[$i]");
   }

   # copy movies into Website-Folder:
   mkdir("../Website/movies",0755);
   foreach(@AVI)
   {
      copy("../$_","../Website/movies/$_");
   }
}

# create the html-files using the corresponding templates
sub create_html
{
   open(F_TEMPLATE,"./templates/index.tplt") || die "error in opening template file for index.html";
   open(F_HTML,">../Website/index.html") || die "could not create the index-file";

   TEMPLATE_FIRST_HALF:
   while (<F_TEMPLATE>)
   {
      last TEMPLATE_FIRST_HALF if ($_ =~ /-+PERL/);
      if ( $_ =~  /---/ )
      {
         $_ =~ s/---TITLE1---/$TITLE1/;
         $_ =~ s/---TITLE2---/$TITLE2/;
         $_ =~ s/---WIDTH---/$WIDTH_SUBST/;
      }
      print F_HTML $_;
   }

   $PAGE_NO = 1;
   print F_HTML "      <td>\n";

   if ( $jpg_index > 1 )
   # Pictures exist
   {
      print F_HTML "        <div align=\"center\">\n";
      print F_HTML "        <table cellpadding=\"2\" cellspacing=\"2\" border=\"2\" bgcolor=\"#000000\">\n";

      # generate Thumbnail-Pages:
      while($PAGE_NO <= $NR_OF_THUMB_PAGES)
      {
         $PICS_VON = ($PAGE_NO * $PICS_IN_PAGE) - $PICS_IN_PAGE + 1;

         if ($PAGE_NO < $NR_OF_THUMB_PAGES)
            { $PICS_BIS = $PAGE_NO * $PICS_IN_PAGE; }
         else
            { $PICS_BIS = $NR_OF_PICS; }

         # generate Link to Thumbnail-Page in index.html:
         print F_HTML &print_page_row($PAGE_NO, $PICS_VON, $PICS_BIS);

         # generate Tumbnail-Page:
         &create_thumbnail_page($PAGE_NO, $PICS_VON, $PICS_BIS);

         # generate Html-Files of JPG-Pictures:
         for ($i=$PICS_VON; $i<=$PICS_BIS; $i++)
         {
            &create_html_pic_file($i,$PICS_VON,$PICS_BIS,$PAGE_NO)
         }

         $PAGE_NO++;
      }
      print F_HTML "        </table>\n";
      print F_HTML "        </div>\n";
   }
   print F_HTML "      </td>\n";
   print F_HTML "      <td></td>\n";
   print F_HTML "      <td>\n";

   $i = 1;
   # generate Links to avi's:
   if ($avi_index > 0)
   {
      print F_HTML "        <div align=\"center\">\n";
      print F_HTML "        <table cellpadding=\"2\" cellspacing=\"2\" border=\"2\" bgcolor=\"#000000\">\n";

      foreach(@AVI)
      {
         print F_HTML &print_avi_row($i,$_);
         $i++;
      }

      print F_HTML "        </table>\n";
      print F_HTML "        </div>\n";
   }

   print F_HTML "      </td>\n";

   while (<F_TEMPLATE>)
   {
      print F_HTML $_;
   }

   close(F_TEMPLATE);
   close(F_HTML);
}

# generate the link to a thumbnail page in index.html
sub print_page_row
{
   $PAGE_NO  = $_[0];
   $PICS_VON = $_[1];
   $PICS_BIS = $_[2];

   $return_str = "          <tr valign=\"middle\" bgcolor=\"#FFFFCC\">";
   $return_str = $return_str."<td><a href=\"tab$PAGE_NO.html\"> Pictures $PICS_VON - $PICS_BIS </a> </td>";
   $return_str = $return_str."<td><img src=\"thumbs/$JPG_RENAMED[$PICS_VON]\"></td></tr>\n";

   return $return_str;
}

# generate the link to an avi-file in index.html
sub print_avi_row
{
   $i = $_[0];
   $MOVIE_NAME = $_[1];
   $FILE_SIZE = (-s "../Website/movies/$MOVIE_NAME") / 1000000;
   $FILE_SIZE =~ /.*\.../;
   $FILE_SIZE = $&;
   $MOVIE_NAME =~ s/.[Aa][vV][iI]//;
   $MOVIE_NAME =~ s/^\d+//;

   $return_str = "          <tr valign=\"middle\" bgcolor=\"#99FFCC\">";
   $return_str = $return_str."<td>Movie $i:&nbsp;&nbsp;</td>";
   $return_str = $return_str."<td colspan=2> <a href=\"./movies/$_[1]\">$MOVIE_NAME</a> &nbsp;&nbsp;&nbsp;($FILE_SIZE MB) </td></tr>\n";

   return $return_str;
}

# create a thumbnail-page
sub create_thumbnail_page
{
   $PAGE_NO  = $_[0];
   $PICS_VON = $_[1];
   $PICS_BIS = $_[2];

   $BACK       = $PAGE_NO -1;
   $FORWARD    = $PAGE_NO + 1;

   if ($FORWARD <= $NR_OF_THUMB_PAGES)
   { $FORWARD_PAGE = "tab$FORWARD.html"; }
   else
   { $FORWARD_PAGE = "index.html" }

   if ($BACK > 0)
   { $BACK_PAGE = "tab$BACK.html"; }
   else
   { $BACK_PAGE = "index.html" }


   open(F_THUMB_TEMPLATE,"./templates/thumb_menu.tplt") || die "error in opening template file for tab$PAGE_NO.html";
   open(F_THUMB_HTML,">../Website/tab$PAGE_NO.html") || die "could not create the tab$PAGE_NO-file";

   TEMPLATE_FIRST_HALF:
   while (<F_THUMB_TEMPLATE>)
   {
      if ($_ =~ /---/)
      {
         $_ =~ s/---FORWARD---/$FORWARD_PAGE/;
         $_ =~ s/---BACK---/$BACK_PAGE/;
         $_ =~ s/---TITLE---/Picture Page $PAGE_NO/;
      }

      last TEMPLATE_FIRST_HALF if ($_ =~ /-+PERL/);
      print F_THUMB_HTML $_;
   }

   print F_THUMB_HTML &create_thumb_table($PICS_VON);

   while (<F_THUMB_TEMPLATE>)
   {
      if ($_ =~ /---/)
      {
         $_ =~ s/---FORWARD---/$FORWARD_PAGE/;
         $_ =~ s/---BACK---/$BACK_PAGE/;
         $_ =~ s/---TITLE---/Picture Page $PAGE_NO/;
      }
      print F_THUMB_HTML $_;
   }

   close(F_THUMB_TEMPLATE);
   close(F_THUMB_HTML);
}

# create the thumnail-table within the thumnail-page
sub create_thumb_table
{
   $RETURN_STRING = "";
   $PIC_NO = $_[0];

   SECOND_STOP:
   for($i = 0; $i < $NUMBER_ROWS; $i++)
   {
      $RETURN_STRING = $RETURN_STRING."   <tr>\n";

      FIRST_STOP:
      for($f = 0; $f < $NUMBER_COLS; $f++)
      {
         $RETURN_STRING = $RETURN_STRING."       <td><div align=\"center\"><a href=\"./web/$HTML_PIC[$PIC_NO]\"><img src=\"./thumbs/$JPG_RENAMED[$PIC_NO]\" border=1 align=\"middle\"></a></div></td>\n";
         $PIC_NO++;
         last FIRST_STOP if ($PIC_NO > $NR_OF_PICS);
      }

      $RETURN_STRING = $RETURN_STRING."   </tr>\n";
      last SECOND_STOP if ($PIC_NO > $NR_OF_PICS);
   }

   return $RETURN_STRING;
}

# create the html-page for one picture
sub create_html_pic_file
{
   $PIC_NO    = $_[0];
   $PICS_VON  = $_[1];
   $PICS_BIS  = $_[2];
   $PAGE_NO   = $_[3];

   $MAIN       = "../tab$PAGE_NO.html";

   if ($PIC_NO < $PICS_BIS)
   { $FORWARD_PAGE = $HTML_PIC[$PIC_NO + 1]; }
   else
   { $FORWARD_PAGE = $MAIN }

   if ($PIC_NO > $PICS_VON)
   { $BACK_PAGE = $HTML_PIC[$PIC_NO - 1]; }
   else
   { $BACK_PAGE = $MAIN }

   $TITLE = $JPG[$PIC_NO];
   $TITLE =~ s/\.[jJ][pP].?[gG]$//;
   $TITLE =~ s/^\d+//;

   open(F_HTML_PIC_TEMPLATE,"./templates/html_pic.tplt") || die "error in opening template file for $HTML_PIC[$PIC_NO]";
   open(F_HTML_PIC_FILE,">../Website/web/$HTML_PIC[$PIC_NO]") || die "could not create the $HTML_PIC[$PIC_NO] file";

   TEMPLATE_FIRST_HALF:
   while (<F_HTML_PIC_TEMPLATE>)
   {
      if ($_ =~ /---/)
      {
         $_ =~ s/---BACK---/$BACK_PAGE/;
         $_ =~ s/---MAIN---/$MAIN/;
         $_ =~ s/---FORWARD---/$FORWARD_PAGE/;
         $_ =~ s/---TITLE---/$TITLE/;
         $_ =~ s/---WAIT_TIME---/$WAIT_TIME/;
      }

      last TEMPLATE_FIRST_HALF if ($_ =~ /-+PERL/);
      print F_HTML_PIC_FILE $_;
   }

   # only generate the link to the original picture, if INCLUDE_ORIG is configured to 'TRUE'

   if ($INLCUDE_ORIG =~ /TRUE/ )
   {
      print F_HTML_PIC_FILE  "     <a href=\"../pics/$JPG_RENAMED[$PIC_NO]\"><img src=\"./$JPG_RENAMED[$PIC_NO]\"></a>\n";
      $INCLUDE_ORIG_TEXT = "<div align=\"center\">click on pic to see the fullsized image </div>";
   }
   else
   {
      print F_HTML_PIC_FILE  "     <a><img src=\"./$JPG_RENAMED[$PIC_NO]\"></a>\n";
      $INCLUDE_ORIG_TEXT = "";
   }

   while (<F_HTML_PIC_TEMPLATE>)
   {
      if ($_ =~ /---/)
      {
         $_ =~ s/---BACK---/$BACK_PAGE/;
         $_ =~ s/---MAIN---/$MAIN/;
         $_ =~ s/---FORWARD---/$FORWARD_PAGE/;
         $_ =~ s/---WAIT_TIME---/$WAIT_TIME/;
         $_ =~ s/---INCLUDE_ORIG_TEXT---/$INCLUDE_ORIG_TEXT/;
      }
      print F_HTML_PIC_FILE $_;
   }

   close(F_HTML_PIC_TEMPLATE);
   close(F_HTML_PIC_FILE);
}

# procedure to copy picture $_[1] to thumbnail $_[2] with size $_[0]
sub copy_to_thumbnail
{
   $SIZE  = $_[0];
   $PIC   = $_[1];
   $THUMB = $_[2];

   # read original pic:
   $image = new Image::Magick;
   $image->Read($PIC);
   $ORIG_WIDTH  = $image->Get('width');
   $ORIG_HEIGHT = $image->Get('height');

   if ($ORIG_WIDTH > $ORIG_HEIGHT)
   {
      $WIDTH = $SIZE;
      $COMPRESS_FACTOR = $ORIG_WIDTH / $SIZE;
      $HEIGHT = int($ORIG_HEIGHT / $COMPRESS_FACTOR);
   }
   else
   {
      $HEIGHT = $SIZE;
      $COMPRESS_FACTOR = $ORIG_HEIGHT / $SIZE;
      $WIDTH = int($ORIG_WIDTH / $COMPRESS_FACTOR);
   }

   #print "Creating Thumb: $THUMB\n";

   # create thumbnail:
   $image->Resize(width=>$WIDTH, height=>$HEIGHT);
   $image->Write($THUMB);
}